/**********************************************************************
File Name:              taskFlyport.c
Dev:                    Daniele De Santis
Email:                  desantis@areasx.com
Web:                    http://www.areasx.com

Versione:               0.0
Data:                   03/06/13
Note Hardware:  
Note Software: 
**********************************************************************/


#include "taskFlyport.h"
#include "sd_manager.h"
#include "time.h"
#include "rtcc.h"

//#################################################################################
//#################################################################################
//Configurazione Parametri LAN
#define   LAN_IP            "192.168.0.137"
#define   LAN_MASK          "255.255.255.0"
#define   LAN_DNS1          "195.110.128.1"
#define   LAN_DNS2          "212.48.4.11"
#define   LAN_GW            "192.168.0.1"

//Configurazione parametri Wi-Fi solo su Ppenpicus Wi-Fi
#define   WI_FI_SSID        "www.areasx.com"
#define   WI_FI_SECURITY    WF_SECURITY_WPA2_WITH_PASS_PHRASE   //Protezione WPA2
//#define   WI_FI_SECURITY    WF_SECURITY_WPA_WITH_PASS_PHRASE  //Protezione WAP
//#define   WI_FI_SECURITY    WF_SECURITY_WEP_104               //Protezione WEP 128
//#define   WI_FI_SECURITY    WF_SECURITY_OPEN                  //Nessuna Protezione WI_FI_PASS_PHRASE ""
#define   WI_FI_PASS_PHRASE  "xxxxxxxxx"

//Configurazione Soglia attivazione Relè funzione Termostato
#define   SOGLIA_MODE      1   //0 nessuna azione; 1 Superata soglia Temp; 2 superata soglia Umid; 3 sotto soglia Temp; 2 sotto soglia Umid;
#define   SOGLIA_VALUE     25  //Valore soglia Temp o Umid

#define   INTERVALLO_ACQUISIZIONE    120 //Intervallo di acquisizione temp ed umidità in Secondi 60==1min 3600==1 ora
//#################################################################################
//#################################################################################

//LED STATUS (VERDE)
#define   STATUS_LED_ON         IOPut(D5Out , 1)   //OUT4 Net su flyport
#define   STATUS_LED_OFF        IOPut(D5Out , 0)
#define   STATUS_LED_TOGGLE     IOPut(D5Out , IOGet(D5Out) ^ 0x01)


//LED (ROSSO)
#define   ERROR_LED_ON         IOPut(p4 , 1)  //IOPut(D5Out , 1)   //OUT5 Net su flyport
#define   ERROR_LED_OFF        IOPut(p4 , 0) //IOPut(D5Out , 0)
#define   ERROR_LED_TOGGLE     IOPut(p4 , IOGet(p4) ^ 0x01) //IOPut(D5Out , IOGet(D5Out) ^ 0x01)

//RELE' 
#define   RELE_ON         IOPut(p6 , 1);   //Relè su P6
#define   RELE_OFF        IOPut(p6, 0);
#define   RELE_TOGGLE     IOPut(p6, IOGet(p6) ^ 0x01)
#define   RELE_STATE      IOGet(p6)

//PULSANTE
#define   BUTTON_INIT          IOInit(p2,inup)
#define   BUTTON_PRESS         (IOGet(p2)==0)

//DEFINE GPIO SHT75
#define DATA_LINE         p3
#define CLOCK_LINE        p1
#define DATA_LINE_IN      IOInit(DATA_LINE,inup)
#define DATA_LINE_OUT     IOInit(DATA_LINE,0)
#define DATA_LINE_LOW     DATA_LINE_OUT;IOPut(DATA_LINE,0)
#define DATA_LINE_HIGH    DATA_LINE_OUT;IOPut(DATA_LINE,1) 
#define DATA_LINE_READ    IOGet(DATA_LINE)
#define CLOCK_LINE_OUT    IOInit(CLOCK_LINE,0)
#define CLOCK_LINE_LOW    IOPut(CLOCK_LINE,0)
#define CLOCK_LINE_HIGH   IOPut(CLOCK_LINE,1)
#define SHT_TIMEOUT       1
#define CMD_READ_TEMP     0x03
#define CMD_READ_HUM      0x05
#define CMD_RESET         0x1e



extern BOOL  filelock;
double       temperature=0;
double       humidity=0;


//PROTOTIPI
void    SHT_SendStart(void) ;
void    SHT_SendReset (void);
int     SHT_SendByte(char byte);
char    SHT_ReadByte(int withack);
double  SHT_ReadTemperature(void);
double  SHT_ReadHumidity(void);


void SHT_SendStart(void) {
  DATA_LINE_OUT;
  CLOCK_LINE_HIGH;
  vTaskDelay(1);
  DATA_LINE_IN;
  DATA_LINE_LOW;
  CLOCK_LINE_LOW;
  CLOCK_LINE_HIGH;
  vTaskDelay(1);
  DATA_LINE_HIGH;
  CLOCK_LINE_LOW;
  DATA_LINE_IN;
}
void SHT_SendReset (void) {
  int i;
  CLOCK_LINE_OUT;
  for (i=0;i<12;i++) {
    CLOCK_LINE_HIGH;
    vTaskDelay(1);
    CLOCK_LINE_LOW;
    vTaskDelay(1);
  }
}
int SHT_SendByte(char byte) {
  char tempbyte;
  int i;
  DWORD t0;
  DATA_LINE_OUT;
  tempbyte=byte;
  for (i=0x80;i>0;i/=2) {
    if (tempbyte & i) {
      DATA_LINE_HIGH;
    } else {
      DATA_LINE_LOW;
    }
    CLOCK_LINE_HIGH;
    vTaskDelay(1);
    CLOCK_LINE_LOW;
  }
  DATA_LINE_IN;
  CLOCK_LINE_HIGH;
  t0=TickGet();
  while (DATA_LINE_READ) {
    if((TickGet()-t0)> SHT_TIMEOUT*TICK_SECOND) {
      SHT_SendReset();
      return 0;
    }
  }
  CLOCK_LINE_LOW;
  return 1;
}

char SHT_ReadByte(int withack) {
  char tempbyte;
  int i;
  tempbyte=0;
  DATA_LINE_IN;
  for (i=0x80;i>0;i/=2) {
    CLOCK_LINE_HIGH;
    vTaskDelay(1);
    if (DATA_LINE_READ) tempbyte |= i;
    CLOCK_LINE_LOW;
  }  
  vTaskDelay(1);
  if (withack) {
    DATA_LINE_OUT;
    DATA_LINE_LOW;
    CLOCK_LINE_HIGH;
    vTaskDelay(1);
    CLOCK_LINE_LOW;
    DATA_LINE_IN;
  } else {
    DATA_LINE_OUT;
    DATA_LINE_HIGH;
    CLOCK_LINE_HIGH;
    vTaskDelay(1);
    CLOCK_LINE_LOW;
    DATA_LINE_IN;
  }
  return tempbyte;
}

double SHT_ReadTemperature(void) {
  char data[2];
  char chk;
  unsigned int itemp;
  DWORD t0;
  SHT_SendStart();
  if (!SHT_SendByte(CMD_READ_TEMP)) return -101;
  t0=TickGet();
  while (DATA_LINE_READ) {
    if((TickGet()-t0)> SHT_TIMEOUT*TICK_SECOND) {
      SHT_SendReset();
      return -101;
    }
  }
  data[0]=(char)SHT_ReadByte(1);
  data[1]=(char)SHT_ReadByte(1);
  chk=SHT_ReadByte(0);
  itemp=((int)data[0]<<8)+((unsigned char)data[1] & 0xFF);
  return ((0.01*((float)itemp))-40.00);
}

double SHT_ReadHumidity(void) { 
  char data[2];
  char chk;
  int itemp;
  DWORD t0;
  SHT_SendStart();
  if (!SHT_SendByte(CMD_READ_HUM)) return -101;
  t0=TickGet();
  while (DATA_LINE_READ) {
    if((TickGet()-t0)> SHT_TIMEOUT*TICK_SECOND) {
      SHT_SendReset();
      return -101;
    }
  }
  data[0]=(char)SHT_ReadByte(1);
  data[1]=(char)SHT_ReadByte(1);
  chk=SHT_ReadByte(0);
  itemp=((int)data[0]<<8)+((unsigned char)data[1] & 0xFF);
  return ((-4)+(0.0405*(float)itemp)+(-0.0000028*((float)itemp*(float)itemp)));
}

BYTE LegalTime(struct tm* ts){
  if(ts->tm_mon==9){
    if(ts->tm_wday==0){
      if((ts->tm_mday+7)>31) {
        if(ts->tm_hour>=2) return 0;
      }
      return 1;
    } else {
      if((ts->tm_mday + (7 - ts->tm_wday))> 31){
        return 0;
      }
      return 1;
    }
  } else if(ts->tm_mon==2) {
    if(ts->tm_wday==0){
      if((ts->tm_mday+7)>31) {
        if(ts->tm_hour>=1) return 1; 
      }
      return 0;
    } else {
      if((ts->tm_mday + (7 - ts->tm_wday))> 31){ 
        return 1;
      }
      return 0;
    }
  } else if(ts->tm_mon>=3 && ts->tm_mon<=8) { 
    return 1;
  }
  return 0;
}

/*****************************
******************************/

void FlyportTask() {
  char debug[200];
  BOOL repeatInit = TRUE;
  BOOL initRes = FALSE;
  BOOL resultSD = FALSE;
  int error = 0;
  DWORD free;
  DWORD        timer0=0;
  DWORD        timer1=0;
  struct       tm mytime;
  
  DWORD epoch=0;
  DWORD epochtime=0xA2C2A;
  time_t now;
  struct tm* ts;
  int GMT_zone = 1;

  IOInit(p19, inup); // SD-CS line
  BUTTON_INIT;
  RELE_OFF;
  
  mytime.tm_hour =15;
  mytime.tm_min = 55;
  mytime.tm_sec = 00;
  mytime.tm_mday = 7;
  mytime.tm_mon = 5;
  mytime.tm_year = 113; //attuale -2000
  mytime.tm_wday = SUNDAY;
  RTCCSet(&mytime);
  
  UARTWrite(1,"Igro Termo Logger!\r\n");
  
  NETSetParam(MY_IP_ADDR,      LAN_IP);
  NETSetParam(MY_IP_ADDR,      LAN_IP);
  NETSetParam(SUBNET_MASK,     LAN_MASK);
  NETSetParam(PRIMARY_DNS,     LAN_DNS1);
  NETSetParam(SECONDARY_DNS,   LAN_DNS2);
  NETSetParam(MY_GATEWAY,      LAN_GW);
  NETSetParam(DHCP_ENABLE,     DISABLED);
  
  NETSetParam(SSID_NAME,        WI_FI_SSID);
  NETSetParam(NETWORK_TYPE,     INFRASTRUCTURE);
  WFSetSecurity(WI_FI_SECURITY, WI_FI_PASS_PHRASE, strlen(WI_FI_PASS_PHRASE),0);
  WFConnect(WF_CUSTOM);


  SHT_SendReset();
  
  while(1){
    if (WFStatus != CONNECTED){  //Attendo connessione Wi-Fi
        ERROR_LED_TOGGLE;
        vTaskDelay(100);
    } else {
      ERROR_LED_OFF;
      STATUS_LED_TOGGLE;
      

      //Inizializzazione RTC trmite server NTP (richiesta connessione internet)
      if((timer0==0)||(TickGet() - timer0 > (3600*TICK_SECOND))){
        timer0=TickGet();
        UARTWrite(1,"Sincronizzazione NTP\r\n");
        while(epoch<epochtime) {
          vTaskDelay(50);
          epoch=SNTPGetUTCSeconds(); 			//Abilitare SNTP Client nel wizard.
        }
        now =(time_t)epoch;
        ts  = localtime(&now);
        ts->tm_hour = (ts->tm_hour + GMT_zone + LegalTime(ts));
        if(ts->tm_hour > 24) {
          ts->tm_hour = ts->tm_hour - 24;
        } else if(ts->tm_hour < 0) {
          ts->tm_hour = ts->tm_hour + 24;
        }
        RTCCSet(ts);
        UARTWrite(1,"Sincronizzazione NTP OK\r\n");
      }
      
      if((timer1==0) || (TickGet() - timer1 > (INTERVALLO_ACQUISIZIONE*TICK_SECOND))){
        timer1=TickGet();
        
        while(filelock){ //Blocco se il file e' in download
          STATUS_LED_TOGGLE; 
          vTaskDelay(10); 
        }
        
        //Acquisizione da sonda SHT75
        temperature= SHT_ReadTemperature();
        humidity   = SHT_ReadHumidity();
        if(temperature==-101 || humidity==-101){
          UARTWrite(1, "ERRORE - sonda SHT75\n");
          ERROR_LED_ON;
          vTaskDelay(1000);
          timer1=0;
          continue;
        }
        
        //Controllo Relè
        switch(SOGLIA_MODE){
          case 0: //Nessuna azione
          break;
          case 1: //Superata Tmax
            if(temperature>SOGLIA_VALUE) {
              RELE_ON;
            } else {
              RELE_OFF;
            }
          break;
          case 2: //Superata Hmax
            if(humidity>SOGLIA_VALUE) {
              RELE_ON;
            } else {
              RELE_OFF;
            }
          break;
          case 3: //Superata Tmin
            if(temperature<SOGLIA_VALUE) {
              RELE_ON;
            } else {
              RELE_OFF;
            }
          break;
          case 4: //Superata Hmin
            if(humidity<SOGLIA_VALUE) {
              RELE_ON;
            } else {
              RELE_OFF;
            }
          break;
        }
        
        
        // Check for SD-Card detect signal
        if(IOGet(p19) == TRUE) {
          if(repeatInit == FALSE) {
            UARTWrite(1, "ATTENZIONE UnMount della SD Card\n");
            resultSD = SDUnMount();
            repeatInit = TRUE;
          }
        } else {
          if(repeatInit == TRUE) {
            UARTWrite(1, "Inizializzazione SD Card\n");
            initRes = SDInit(p18, p7, p17, p9, p19, 15);
            if (initRes) {
              repeatInit = FALSE;
              UARTWrite(1,"SD Card inizializzata\n");
            } else {
              error = SDGetErr();
              if (error == SD_NOT_PRESENT) {
                UARTWrite(1, "ERRORE - SD Card non presente\n");
              } else if (error == SD_FS_NOT_INIT) {
                UARTWrite(1, "ERRORE - SD Card non inizializzata\n");
              }
              ERROR_LED_ON;
              vTaskDelay(1000);
              timer1=0;
              continue;
            }
          }
          
          free=SDFreeSpace();
          sprintf(debug, "Spazio free SD %lu\n", free);
          UARTWrite(1, debug);
          if(free>100) {
            if(SDFileCheck("letture.log")) { //File gia' presente
              RTCCGet(&mytime);	
              sprintf ( debug, "%02d/%02d/%04d;%02d:%02d:%02d;%.02fC;%.02f%%;\r\n", mytime.tm_mday, (mytime.tm_mon+1), (1900+ mytime.tm_year), mytime.tm_hour, mytime.tm_min, mytime.tm_sec,temperature, humidity);
              UARTWrite(1,debug);
              if(!SDFileAppend("letture.log", debug, strlen(debug))){
                UARTWrite(1, "ERRORE - Scrittura su SD Card fallita\n");
              }
            } else {
              if(SDFileCreate("letture.log")){
                sprintf(debug, "DATA;ORA;TEMPERATURA;UMIDITA';\r\n");
                if(SDFileAppend("letture.log", debug, strlen(debug))){
                  UARTWrite(1, "ERRORE - Scrittura su SD Card fallita\n");
                }
              } else 
                UARTWrite(1, "ERRORE - Impossibile creare il file sulla SD Card\n");
            }
          } else {
            sprintf(debug, "ERRORE - Spazio sulla SD Card esaurito\n");
            UARTWrite(1, debug);
          }
        }
      }
      
      if(BUTTON_PRESS){  //Se premuto il pulsante sulla SXFLY
        if(SDFileDelete("letture.log"))
          UARTWrite(1, "ATTENZIONE - File letture.log Canellato!\n");
      }
      vTaskDelay(100);
    }
  }
}
